﻿Imports System.ComponentModel.Design
Imports EventBindings.Extensions

<Editor(GetType(Design.EventBindingTypeEditor), GetType(System.Drawing.Design.UITypeEditor))>
<TypeConverter(GetType(EventBindingConverter))>
Public Class EventBinding
    Implements ILifeObservableContainer
    Implements ILifeObservationPublisher

#Region "Fields"
    Private _Event As EventData
    Private _Handling As HandlingData
    Private _Source As IEventBindingSource
    Private _Mediation As MediationData
    Private _Binded As Boolean
#End Region

#Region "Constructors"
    Public Sub New(ByVal eventName As String, ByVal eventSource As Object)
        Me.New(New EventData(eventName, eventSource))
    End Sub
    Public Sub New(ByVal [event] As EventData)
        If Not [event].IsValidEvent Then Throw New InvalidEventException([event])
        Me._Binded = False
        Me._Event = [event]
        Me._Handling = New HandlingData
        Me._Mediation = New MediationData
    End Sub
    Public Sub New(ByVal source As IEventBindingSource,
                   ByVal [event] As EventData, ByVal handling As HandlingData,
                   ByVal mediation As MediationData)
        If source Is Nothing Then Throw New ArgumentNullException("source")
        If Not [event].IsValidEvent Then Throw New InvalidEventException([event])
        If handling Is Nothing OrElse Not handling.CanCreateHandler Then Throw New InvalidHandlerException(handling)
        If mediation Is Nothing OrElse Not mediation.CanCreateMediator([event], handling) Then Throw New InvalidMediatorException(mediation)

        Me._Binded = False
        Me._Mediation = mediation
        Me._Handling = handling
        Me._Event = [event]
    End Sub
#End Region

#Region "Properties"
    <Browsable(False)>
    Public ReadOnly Property [ReadOnly] As Boolean
        Get
            Return Me._Binded
        End Get
    End Property
    <Editor(GetType(Design.EventBindingSourceTypeEditor), GetType(System.Drawing.Design.UITypeEditor))>
    Public Property Source As IEventBindingSource
        Get
            Return Me._Source
        End Get
        Set(ByVal value As IEventBindingSource)
            If Me._Source IsNot Nothing Then Me._Source.Remove(Me)
            Me._Source = value
            If Me._Source IsNot Nothing Then Me._Source.Add(Me)
        End Set
    End Property
    <Browsable(False)>
    Public ReadOnly Property [Event] As EventData
        Get
            Return Me._Event
        End Get
    End Property
    <Editor(GetType(Design.HandlingDataTypeEditor), GetType(System.Drawing.Design.UITypeEditor))>
    <TypeConverter(GetType(ExpandableObjectConverter))>
    Public Property Handling As HandlingData
        Get
            Return Me._Handling
        End Get
        Set(ByVal value As HandlingData)
            If Me.ReadOnly Then Throw New ReadOnlyException
            Me._Handling = value
        End Set
    End Property
    <Editor(GetType(Design.MediationDataTypeEditor), GetType(System.Drawing.Design.UITypeEditor))>
    <TypeConverter(GetType(ExpandableObjectConverter))>
    Public Property Mediation As MediationData
        Get
            Return Me._Mediation
        End Get
        Set(ByVal value As MediationData)
            If Me.ReadOnly Then Throw New ReadOnlyException
            Me._Mediation = value
        End Set
    End Property
    <Browsable(False)>
    Public ReadOnly Property Binded As Boolean
        Get
            Return Me._Binded
        End Get
    End Property
#End Region

#Region "Methods"
    Public Function CanBind() As Boolean
        If Not Me._Event.IsValidEvent Then Return False
        If Me._Handling Is Nothing OrElse Not Me._Handling.CanCreateHandler Then Return False
        If Me._Mediation Is Nothing OrElse Not Me.Mediation.CanCreateMediator(Me._Event, Me._Handling) Then Return False
        Return True
    End Function
    Public Sub BindHandler()
        If Me._Handling.NeedHandlerCreation Then
            Me.NotifyCreating(Of HandlingData)()
            Me._Handling.CreateHandler()
            Me.NotifyCreated(Of HandlingData)(Me._Handling)
        End If
        If Me._Mediation.NeedMediatorCreation Then
            Me.NotifyCreating(Of MediationData)()
            Me._Mediation.CreateMediator(Me._Event, Me._Handling)
            Me.NotifyCreated(Of MediationData)(Me._Mediation)
        End If
        Me._Mediation.Mediator.AddHandler()
        Me._Binded = True
    End Sub
    Public Sub UnbindHandler()
        If Me._Mediation.Mediator IsNot Nothing Then
            Me.NotifyDisposing(Of MediationData)(Me._Mediation)
            Me._Mediation.Mediator.RemoveHandler()
            Me._Mediation.ClearMediator()
            Me.NotifyDisposed(Of MediationData)()
        End If
        Me.NotifyDisposing(Of HandlingData)(Me._Handling)
        Me._Handling.ClearHandler()
        Me.NotifyDisposed(Of HandlingData)()
        Me._Binded = False
    End Sub
    Public Function GetName(ByVal referenceService As IReferenceService) As String
        If Me._Handling Is Nothing Then Return String.Empty
        Return Me._Handling.GetName(referenceService)
    End Function

#End Region

#Region "Notifications"
    Private Function GetLifeObservers(Of TSubject)() As System.Collections.Generic.IEnumerable(Of ILifeObserver(Of TSubject)) Implements ILifeObservableContainer.GetLifeObservers
        Dim Candidates As New List(Of ILifeObserver(Of TSubject))
        If Me._Handling IsNot Nothing Then
            Candidates.AddDistinctRange(Me._Handling.GetLifeObservers(Of TSubject))
        End If
        If Me._Mediation IsNot Nothing Then
            Candidates.AddDistinctRange(Me._Mediation.GetLifeObservers(Of TSubject))
        End If
        If Me._Source IsNot Nothing AndAlso TypeOf Me._Source Is ILifeObserver(Of TSubject) Then
            Candidates.AddDistinct(DirectCast(Me._Source, ILifeObserver(Of TSubject)))
        End If
        Return Candidates
    End Function

    Private Sub NotifyCreating(Of TSubject)() Implements ILifeObservationPublisher.NotifyCreating
        Me.GetLifeObservers(Of TSubject).NotifyCreating(Me)
    End Sub
    Private Sub NotifyCreated(Of TSubject)(ByVal subject As TSubject) Implements ILifeObservationPublisher.NotifyCreated
        Me.GetLifeObservers(Of TSubject).NotifyCreated(Me, subject)
    End Sub
    Private Sub NotifyDisposing(Of TSubject)(ByVal subject As TSubject) Implements ILifeObservationPublisher.NotifyDisposing
        Me.GetLifeObservers(Of TSubject).NotifyDisposing(Me, subject)
    End Sub
    Private Sub NotifyDisposed(Of TSubject)() Implements ILifeObservationPublisher.NotifyDisposed
        Me.GetLifeObservers(Of TSubject).NotifyDisposed(Me)
    End Sub
#End Region

    
End Class
